@@ -88,11 +88,11 @@ class MicroPoll(MicroWriter):
         data.update({'elapsed after downtime':time.time()-start_time})
 
     def maybe_bolster_balance_by_mining(self):
-        """ Mine just a littel to avoid stream dying due to bankruptcy """
+        """ Mine just a little to avoid stream dying due to bankruptcy """
         balance = self.get_balance()
         if balance < 0:
             muid_time = time.time()
-            key = self.bolster_balance_by_mining(seconds=max(50, int(abs(balance) / 10)))
+            key = self.bolster_balance_by_mining(seconds=max(5, int(abs(balance) / 10)))
             mining_time = time.time() - muid_time
             self.mining_time += mining_time
             if key:
@@ -100,6 +100,8 @@ class MicroPoll(MicroWriter):
                 print('     FOUND MUID !!!     ')
                 print('************************', flush=True)
                 self.mining_success += 1
+            else:
+                print('Did not find MUID this time', flush=True)
 
     def run(self):
         data = {'type': 'scheduler', 'scheduler start time': time.time()}
