@@ -22,7 +22,7 @@ def donate1(difficulty=None, password=None, donor='anonymous'):
         except:
             raise Exception("Ask Peter for the donation password ")
     write_key = new_key(difficulty=8)
-    res = requests.post('https://www.microprediction.com/donations/' + write_key,data={'password': password, 'donor': donor})
+    res = requests.post('https://api.microprediction.org/donations/' + write_key,data={'password': password, 'donor': donor})
     if res.status_code==200:
         if 'password' in res.json()['message']:
             return {"error":res.json()}
@@ -33,11 +33,11 @@ def donate1(difficulty=None, password=None, donor='anonymous'):
                 print("Mining and donating the MUIDs with password "+password+" and donor name "+donor+". Thanks. Difficulty set to "+str(difficulty),flush=True)
                 write_key = new_key(difficulty=12)
                 print(write_key,flush=True)
-                res = requests.post('https://www.microprediction.com/donations/' + write_key, data={'password': password,'donor':donor})
+                res = requests.post('https://api.microprediction.org/donations/' + write_key, data={'password': password,'donor':donor})
                 pprint(res.json())
     else:
         return res
 
 
 if __name__=="__main__":
-    donate(difficulty=8,password='5581cee8a281f4fd8cbe404143017046',donor='test')
\ No newline at end of file
+    donate(difficulty=8,password='5581cee8a281f4fd8cbe404143017046',donor='test')
