@@ -1,6 +1,7 @@
 from microprediction.reader import MicroReader
 from microconventions import api_url
-import requests, pprint
+import requests, pprint, time, json
+from collections import OrderedDict
 
 class MicroWriter(MicroReader):
 
@@ -11,6 +12,8 @@ class MicroWriter(MicroReader):
         assert self.key_difficulty(string_write_key), "Invalid write_key. Mine one at muid.org. "
         self.write_key = string_write_key
         self.verbose   = verbose
+        self.code      = self.shash(write_key)            # Unique identifier that can be shared
+        self.animal    = self.animal_from_key(write_key)  # ... and corresponding spirit animal
 
     def __repr__(self):
         return {'write_key':self.write_key,"animal":self.animal_from_key(self.write_key)}
@@ -69,6 +72,30 @@ class MicroWriter(MicroReader):
         else:
             raise Exception('Failed for '+ self.write_key)
 
+    def delete_errors(self):
+        """ Clear log of errors """
+        res = requests.delete(self.base_url + '/errors/' + self.write_key)
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
+    def get_warnings(self):
+        """ Retrieve private log information """
+        res = requests.get(self.base_url + '/warnings/' + self.write_key)
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
+    def delete_warnings(self):
+        """ Clear warnings """
+        res = requests.delete(self.base_url + '/warnings/' + self.write_key)
+        if res.status_code == 200:
+            return res.json()
+        else:
+            raise Exception('Failed for ' + self.write_key)
+
     def get_balance(self):
         res = requests.get(self.base_url + '/balance/' + self.write_key)
         if res.status_code == 200:
@@ -77,7 +104,7 @@ class MicroWriter(MicroReader):
             raise Exception('Failed for ' + self.write_key)
 
     def put_balance(self, source_write_key, amount=100. ):
-        """ Transfer some balance into self.write_key """
+        """ Transfer some balance into self.write_key by reducing balance of source_write_key """
         res = requests.put(self.base_url + '/balance/' + self.write_key, data={"source_write_key": source_write_key,"amount":amount})
         if res.status_code == 200:
             return res.json()
@@ -95,12 +122,12 @@ class MicroWriter(MicroReader):
 
     def bolster_balance_by_mining(self,seconds=1):
         """ For a short time, try to burn a MUID to bolster flagging balance """
-        key = self.maybe_create_key(difficulty=12,seconds=seconds)
+        key = self.maybe_create_key(difficulty=11,seconds=seconds)
         if key is not None:
             self.put_balance(source_write_key=key)
             return key
 
-    def restore_balance_by_mining(self, difficulty=12):
+    def restore_balance_by_mining(self, difficulty=11):
         """ Mine a MUID and deposit it to bring the balance up from a negative number """
         source_write_key = self.create_key(difficulty=difficulty)
         return self.put_balance(source_write_key=source_write_key)
@@ -108,23 +135,61 @@ class MicroWriter(MicroReader):
     def get_confirms(self):
         res = requests.get(self.base_url + '/confirms/' + self.write_key)
         if res.status_code == 200:
-            return res.json()
+            confirm_strings = res.json()
+            return [json.loads(c) for c in confirm_strings]
         else:
             raise Exception('Failed for ' + self.write_key)
 
-    def get_leaderboard(self,name,delay=None):
-        res = requests.patch(self.base_url + '/leaderboard/' + name, data={"delay": delay})
+    def get_infrequent_confirms(self):
+        """ Everything except stream prediction and updates """
+        COMMON = ['set','submit']
+        return [c for c in self.get_confirms() if c.get('operation') not in COMMON]  # TODO: Move strings like 'withdraw' to microconventions
+
+    def get_withdrawals(self):
+        """ Get withdrawal confirmations """
+        return [ c for c in self.get_confirms() if c.get('operation')=='withdraw'] # TODO: Move strings like 'withdraw' to microconventions
+
+    def get_cancellations(self):
+        """ Get cancellation confirmations, which can occur long after withdrawal requests """
+        return [ c for c in self.get_confirms() if c.get('operation')=='cancel']
+
+    def get_submissions(self):
+        """ Get confirmations of submissions of predictions """
+        return [c for c in self.get_confirms() if c.get('operation') == 'submit']
+
+    def get_set_confirmations(self):
+        """ Get confirmations of set operations """
+        return [c for c in self.get_confirms() if c.get('operation') == 'set']
+
+    def get_elapsed_since_confirm(self):
+        confirms = self.get_confirms()
+        if not confirms:
+            return None
+        last_confirm_time  = confirms[0].get("epoch_time")
+        return time.time()-last_confirm_time
+
+    def get_transactions(self, with_epoch=True):
+        """
+            :param with_timestamps bool     Set true to include epoch time (datetime already comes)
+        """
+        res = requests.get(self.base_url + '/transactions/' + self.write_key)
         if res.status_code == 200:
-            return res.json()
+            transactions = res.json()
+            transaction_values = [t[1] for t in transactions]
+            if with_epoch:
+                transaction_epoch_times = [ int(t[0].split('-')[0])/1000 for t in transactions ]
+                for tv,tt in zip(transaction_values,transaction_epoch_times):
+                    tv.update({'epoch_time':tt})
+            return transaction_values
         else:
             raise Exception('Failed for ' + self.write_key)
 
-    def get_overall(self):
-        res = requests.patch(self.base_url + '/overall/')
-        if res.status_code == 200:
-            performance = res.json()
-        else:
-            raise Exception('Failed for ' + self.write_key)
+    def get_elapsed_since_transaction(self):
+        transactions = self.get_transactions(with_epoch=True)
+        if not transactions:
+            return None
+        last_transaction_time  = transactions[0].get("epoch_time")
+        return time.time()-last_transaction_time
 
     def get_active(self):
         res = requests.get(self.base_url + '/active/' + self.write_key)
@@ -191,3 +256,24 @@ class MicroWriter(MicroReader):
             raise Exception('Failed to cancel.')
         else:
             return success
+
+    def active_performance(self, reverse=False, performance=None, active=None):
+        performance = performance or self.get_performance()
+        active      = active or self.get_active()
+        return OrderedDict(
+            sorted([(horizon, balance) for horizon, balance in performance.items() if horizon in active],
+                   key=lambda e: e[1], reverse=reverse))
+
+    def worst_active_horizons(self, stop_loss, performance=None, active=None):
+        """ Return horizons where we are losing more than stop_loss """
+        return [horizon for horizon, balance in self.active_performance(reverse=True,performance=performance,active=active).items() if
+                balance < -abs(stop_loss)]
+
+    def cancel_worst_active(self, stop_loss, num=1000, performance=None, active=None):
+        horizons = self.worst_active_horizons(stop_loss=stop_loss,performance=performance,active=active)[:num]
+        for horizon in horizons:
+            name, delay = self.split_horizon_name(horizon)
+            self.cancel(name=name, delays=[delay])
+            print('Cancelled participation in '+str(horizon),flush=True)
+            time.sleep(0.1)
+        return horizons
\ No newline at end of file
